# Simple stub demonstrating rci loopback on a ConnectPort
# Only tested on an X2 but includes support for X2e
# XML parsing simplified and naive. For final application should be made 
# more robust, check for errors, etc.

import time

# On the X2 time.clock() counts time since device boot
# On the X2e linux platform time.clock() counts processor time taken by
# the interpreter
# To workaround this difference for timing logic replace time.clock()
# on the X2e
try:
    import uptime
    time.clock = uptime.uptime
except ImportError:
    pass

# On the X2e prefer the non-blocking rci module, plays nicer with other
# threads
try:
    import rci_nonblocking as rci
except ImportError:
    import rci
import traceback

# Templated to control whether the get_version is performed synchronously or
# asynchronously.
# Other commands could be similarly templated for device address, install code,
# etc.
GET_VERSION_TEMPLATE =\
"""\
<rci_request version="1.1"><do_command target="RPC_request">\
<get_version synchronous="%s"/>\
</do_command></rci_request>\
"""

GET_ASYNC_RESPONSES =\
"""\
<rci_request version="1.1"><do_command target="RPC_response">\
<get_responses/>
</do_command></rci_request>\
"""

# Naive XML parser to extract a version string.
# Will break in all sorts of edge cases depending on the input.
def parse_version(xml_text):
    start_token = '<version type="string">'
    version = None
    start = xml_text.find(start_token)
    if start != -1:
        start += len(start_token)
        end = xml_text.find('<', start)
        version = xml_text[start:end]
    return version

# A synchronous request works even when the gateway has an iDigi connection
# and is actively pushing, no interference with normal operation
def get_sef_version_sync():
    result = rci.process_request(GET_VERSION_TEMPLATE % "true")
    return parse_version(result)

# Async requests should only be used in a couple of limited circumstances
# 1) When the gateway is not connected to iDigi (can get responses)
# 2) When the gateway is connected to iDigi and don't care about response
#
# If the gateway is connected to iDigi requesting the response will often
# fail (may not hit before response is pushed to iDigi) or might eat misc
# responses from other tasks (for example periodic reporting) and prevent
# their receipt by iDigi.
def get_sef_version_async(timeout = 5, sleep_time = 0):
    rci.process_request(GET_VERSION_TEMPLATE % "false")
    if sleep_time > 0:
        time.sleep(sleep_time) # used to demo failure to retrieve async responses
    t0 = time.clock()
    version = None
    while version is None and time.clock() - t0 < timeout:
        result = rci.process_request(GET_ASYNC_RESPONSES)
        version = parse_version(result)
    return version

def main_loop():
    print 'Entered main loop'
    print get_sef_version_sync() # will get response
    print get_sef_version_async() # may get response
    print get_sef_version_async(sleep_time = 1) # probably won't get response
    while True:
        raise Exception('Exit for now, replace with state machine as needed')

if __name__ == "__main__":
    main_loop()